@@ -3,12 +3,15 @@
 //!
 //! When making a release of a Rust project, you typically need to
 //! adjust some version numbers in your code and documentation. This
-//! crate gives you macros that covers the two usual cases where
+//! crate gives you macros that covers some typical cases where
 //! version numbers need updating:
 //!
 //! * TOML examples in the `README.md` files that show how to add a
 //!   dependency on your crate. See [`assert_markdown_deps_updated`].
 //!
+//! * A `Changelog.md` file that should at least mention the current
+//!   version. See [`assert_contains_regex`].
+//!
 //! * The [`html_root_url`] attribute that tells other crates where to
 //!   find your documentation. See [`assert_html_root_url_updated`].
 //!
@@ -44,12 +47,14 @@
 //! [`html_root_url`]: https://rust-lang-nursery.github.io/api-guidelines/documentation.html#crate-sets-html_root_url-attribute-c-html-root
 //! [`assert_markdown_deps_updated`]: macro.assert_markdown_deps_updated.html
 //! [`assert_html_root_url_updated`]: macro.assert_html_root_url_updated.html
+//! [`assert_contains_regex`]: macro.assert_contains_regex.html
 
-#![doc(html_root_url = "https://docs.rs/version-sync/0.5.0")]
+#![doc(html_root_url = "https://docs.rs/version-sync/0.6.0")]
 #![deny(missing_docs)]
 
 extern crate itertools;
 extern crate pulldown_cmark;
+extern crate regex;
 extern crate semver_parser;
 extern crate syn;
 extern crate toml;
@@ -59,14 +64,15 @@ use std::fs::File;
 use std::io::Read;
 use std::result;
 
-use pulldown_cmark::{Parser, Event, Tag};
+use itertools::join;
+use pulldown_cmark::{Event, Parser, Tag};
+use regex::Regex;
 use semver_parser::range::parse as parse_request;
-use semver_parser::range::{VersionReq, Op};
-use semver_parser::version::Version;
+use semver_parser::range::{Op, VersionReq};
 use semver_parser::version::parse as parse_version;
+use semver_parser::version::Version;
 use toml::Value;
 use url::Url;
-use itertools::join;
 
 /// The common result type, our errors will be simple strings.
 type Result<T> = result::Result<T, String>;
@@ -127,28 +133,31 @@ fn version_matches_request(version: &Version, request: &VersionReq) -> Result<()
             if pred.major != version.major {
                 return Err(format!(
                     "expected major version {}, found {}",
-                    version.major,
-                    pred.major,
+                    version.major, pred.major,
                 ));
             }
             if let Some(minor) = pred.minor {
                 if minor != version.minor {
-                    return Err(format!("expected minor version {}, found {}",
-                                       version.minor,
-                                       minor));
+                    return Err(format!(
+                        "expected minor version {}, found {}",
+                        version.minor, minor
+                    ));
                 }
             }
             if let Some(patch) = pred.patch {
                 if patch != version.patch {
-                    return Err(format!("expected patch version {}, found {}",
-                                       version.patch,
-                                       patch));
+                    return Err(format!(
+                        "expected patch version {}, found {}",
+                        version.patch, patch
+                    ));
                 }
             }
             if pred.pre != version.pre {
-                return Err(format!("expected pre-release {:?}, found {:?}",
-                                   join(&version.pre, "."),
-                                   join(&pred.pre, ".")));
+                return Err(format!(
+                    "expected pre-release {:?}, found {:?}",
+                    join(&version.pre, "."),
+                    join(&pred.pre, ".")
+                ));
             }
         }
         _ => return Ok(()), // We cannot check other operators.
@@ -165,13 +174,18 @@ fn extract_version_request(pkg_name: &str, block: &str) -> Result<VersionReq> {
                 .get("dependencies")
                 .or_else(|| value.get("dev-dependencies"))
                 .and_then(|deps| deps.get(pkg_name))
-                .and_then(|dep| dep.get("version").or_else(|| Some(dep)))
-                .and_then(|version| version.as_str());
+                .and_then(|dep| {
+                    dep.get("version")
+                        // pkg_name = { version = "1.2.3" }
+                        .and_then(|version| version.as_str())
+                        // pkg_name = { git = "..." }
+                        .or_else(|| dep.get("git").and(Some("*")))
+                        // pkg_name = "1.2.3"
+                        .or_else(|| dep.as_str())
+                });
             match version {
-                Some(version) => {
-                    parse_request(version)
-                        .map_err(|err| format!("could not parse dependency: {}", err))
-                }
+                Some(version) => parse_request(version)
+                    .map_err(|err| format!("could not parse dependency: {}", err)),
                 None => Err(format!("no dependency on {}", pkg_name)),
             }
         }
@@ -258,8 +272,7 @@ fn find_toml_blocks(text: &str) -> Vec<CodeBlock> {
 /// error message. Status information has then already been printed on
 /// `stdout`.
 pub fn check_markdown_deps(path: &str, pkg_name: &str, pkg_version: &str) -> Result<()> {
-    let text = read_file(path)
-        .map_err(|err| format!("could not read {}: {}", path, err))?;
+    let text = read_file(path).map_err(|err| format!("could not read {}: {}", path, err))?;
     let version = parse_version(pkg_version)
         .map_err(|err| format!("bad package version {:?}: {}", pkg_version, err))?;
 
@@ -331,12 +344,11 @@ macro_rules! assert_markdown_deps_updated {
         if let Err(err) = $crate::check_markdown_deps($path, pkg_name, pkg_version) {
             panic!(err);
         }
-    }
+    };
 }
 
 fn url_matches(value: &str, pkg_name: &str, version: &Version) -> Result<()> {
-    let url = Url::parse(value)
-        .map_err(|err| format!("parse error: {}", err))?;
+    let url = Url::parse(value).map_err(|err| format!("parse error: {}", err))?;
 
     // We can only reason about docs.rs.
     if url.domain().is_some() && url.domain() != Some("docs.rs") {
@@ -349,7 +361,8 @@ fn url_matches(value: &str, pkg_name: &str, version: &Version) -> Result<()> {
         return Err(format!("expected \"https\", found {:?}", url.scheme()));
     }
 
-    let mut path_segments = url.path_segments()
+    let mut path_segments = url
+        .path_segments()
         .ok_or_else(|| String::from("no path in URL"))?;
 
     // The package name should not be empty.
@@ -366,7 +379,10 @@ fn url_matches(value: &str, pkg_name: &str, version: &Version) -> Result<()> {
 
     // Finally, we check that the package name and version matches.
     if name != pkg_name {
-        Err(format!("expected package \"{}\", found \"{}\"", pkg_name, name))
+        Err(format!(
+            "expected package \"{}\", found \"{}\"",
+            pkg_name, name
+        ))
     } else {
         // The Rust API Guidelines[1] suggest using an exact version
         // number, but we have relaxed this a little and allow the
@@ -394,46 +410,48 @@ fn url_matches(value: &str, pkg_name: &str, version: &Version) -> Result<()> {
 /// succinct error message. Status information has then already been
 /// printed on `stdout`.
 pub fn check_html_root_url(path: &str, pkg_name: &str, pkg_version: &str) -> Result<()> {
-    let code = read_file(path)
-        .map_err(|err| format!("could not read {}: {}", path, err))?;
+    let code = read_file(path).map_err(|err| format!("could not read {}: {}", path, err))?;
     let version = parse_version(pkg_version)
         .map_err(|err| format!("bad package version {:?}: {}", pkg_version, err))?;
-    let krate = syn::parse_crate(&code)
+    let krate: syn::File = syn::parse_str(&code)
         .map_err(|_| format!("could not parse {}: please run \"cargo build\"", path))?;
 
     println!("Checking doc attributes in {}...", path);
     for attr in krate.attrs {
-        let (ident, nested_meta_items) = match attr {
-            syn::Attribute {
-                style: syn::AttrStyle::Inner,
-                value: syn::MetaItem::List(ref ident, ref nested_meta_items),
-                is_sugared_doc: false,
-            } => (ident, nested_meta_items),
+        if let syn::AttrStyle::Outer = attr.style {
+            continue;
+        }
+        let (ident, nested_meta_items) = match attr.interpret_meta() {
+            Some(syn::Meta::List(syn::MetaList { ident, nested, .. })) => (ident, nested),
             _ => continue,
         };
 
-        if ident.as_ref() != "doc" {
+        if ident != "doc" {
             continue;
         }
 
         for nested_meta_item in nested_meta_items {
-            let meta_item = match *nested_meta_item {
-                syn::NestedMetaItem::MetaItem(ref meta_item) => meta_item,
+            let meta_item = match nested_meta_item {
+                syn::NestedMeta::Meta(ref meta_item) => meta_item,
                 _ => continue,
             };
 
             let check_result = match *meta_item {
-                syn::MetaItem::NameValue(ref name, ref value) if name == "html_root_url" => {
-                    match *value {
+                syn::Meta::NameValue(syn::MetaNameValue {
+                    ref ident, ref lit, ..
+                })
+                    if ident == "html_root_url" =>
+                {
+                    match *lit {
                         // Accept both cooked and raw strings here.
-                        syn::Lit::Str(ref s, _) => url_matches(s, pkg_name, &version),
+                        syn::Lit::Str(ref s) => url_matches(&s.value(), pkg_name, &version),
                         // A non-string html_root_url is probably an
                         // error, but we leave this check to the
                         // compiler.
                         _ => continue,
                     }
                 }
-                syn::MetaItem::Word(ref name) if name == "html_root_url" => {
+                syn::Meta::Word(ref name) if name == "html_root_url" => {
                     Err(String::from("html_root_url attribute without URL"))
                 }
                 _ => continue,
@@ -445,7 +463,8 @@ pub fn check_html_root_url(path: &str, pkg_name: &str, pkg_version: &str) -> Res
             //
             // We know such a line must exist since we would have
             // continue the loop above if it wasn't present.
-            let (line_no, source_line) = code.lines()
+            let (line_no, source_line) = code
+                .lines()
                 .enumerate()
                 .find(|&(_, line)| line.contains("html_root_url"))
                 .expect("html_root_url attribute not present");
@@ -519,9 +538,132 @@ macro_rules! assert_html_root_url_updated {
         if let Err(err) = $crate::check_html_root_url($path, pkg_name, pkg_version) {
             panic!(err);
         }
+    };
+}
+
+/// Check that `path` contain the regular expression given by
+/// `template`.
+///
+/// The placeholders `{name}` and `{version}` will be replaced with
+/// `pkg_name` and `pkg_version`, if they are present in `template`.
+/// It is okay if `template` do not contain these placeholders.
+///
+/// The matching is done in multi-line mode, which means that `^` in
+/// the regular expression will match the beginning of any line in the
+/// file, not just the very beginning of the file.
+///
+/// # Errors
+///
+/// If the regular expression cannot be found, an `Err` is returned
+/// with a succinct error message. Status information has then already
+/// been printed on `stdout`.
+pub fn check_contains_regex(
+    path: &str,
+    template: &str,
+    pkg_name: &str,
+    pkg_version: &str,
+) -> Result<()> {
+    // Expand the optional {name} and {version} placeholders in the
+    // template. This is almost like
+    //
+    //   format!(template, name = pkg_name, version = pkg_version)
+    //
+    // but allows the user to leave out unnecessary placeholders.
+    let orig_regex = template
+        .replace("{name}", pkg_name)
+        .replace("{version}", pkg_version);
+
+    // We start by constructing a Regex from the original string. This
+    // ensurs that any errors refer to the string the user passed
+    // instead of the string we use internally.
+    let re = match Regex::new(&orig_regex) {
+        Ok(_) => {
+            // We now know that the regex is valid, so we can enable
+            // multi-line mode by prepending "(?m)".
+            let regex = String::from("(?m)") + &orig_regex;
+            Regex::new(&regex).unwrap()
+        }
+        Err(err) => return Err(format!("could not parse template: {}", err)),
+    };
+    let text = read_file(path).map_err(|err| format!("could not read {}: {}", path, err))?;
+
+    println!("Searching for {:?} in {}...", orig_regex, path);
+    match re.find(&text) {
+        Some(m) => {
+            let line_no = text[..m.start()].lines().count();
+            println!("{} (line {}) ... ok", path, line_no + 1);
+            Ok(())
+        }
+        None => Err(format!("could not find {:?} in {}", orig_regex, path)),
     }
 }
 
+/// Assert that versions numbers are up to date via a regex.
+///
+/// This macro allows you verify that the current version number is
+/// mentioned in a particular file, such as a changelog file. You do
+/// this by specifying a regular expression which will be matched
+/// against the file.
+///
+/// The macro calls [`check_contains_regex`] on the file name given.
+/// The package name and current package version is automatically
+/// taken from the `$CARGO_PKG_NAME` and `$CARGO_PKG_VERSION`
+/// environment variables. These environment variables are
+/// automatically set by Cargo when compiling your crate.
+///
+/// # Usage
+///
+/// The typical way to use this macro is from an integration test:
+///
+/// ```rust
+/// #[macro_use]
+/// extern crate version_sync;
+///
+/// #[test]
+/// # fn fake_hidden_test_case() {}
+/// # // The above function ensures test_readme_mentions_version is
+/// # // compiled.
+/// fn test_readme_mentions_version() {
+///     assert_contains_regex!("README.md", "^### Version {version}");
+/// }
+///
+/// # fn main() {
+/// #     test_readme_mentions_version();
+/// # }
+/// ```
+///
+/// Tests are run with the current directory set to directory where
+/// your `Cargo.toml` file is, so this will find a `README.md` file
+/// next to your `Cargo.toml` file. It will then check that there is a
+/// heading mentioning the current version of your crate.
+///
+/// The regular expression can contain placeholders which are replaced
+/// before the regular expression search begins:
+///
+/// * `{version}`: the current version number of your package.
+/// * `{name}`: the name of your package.
+///
+/// This way you can search for things like `"Latest version of {name}
+/// is: {version}"` and make sure you update your READMEs and
+/// changelogs consistently.
+///
+/// # Panics
+///
+/// If the regular expression cannot be found, `panic!` will be
+/// invoked and your integration test will fail.
+///
+/// [`check_contains_regex`]: fn.check_contains_regex.html
+#[macro_export]
+macro_rules! assert_contains_regex {
+    ($path:expr, $format:expr) => {
+        let pkg_name = env!("CARGO_PKG_NAME");
+        let pkg_version = env!("CARGO_PKG_VERSION");
+        if let Err(err) = $crate::check_contains_regex($path, $format, pkg_name, pkg_version) {
+            panic!(err);
+        }
+    };
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -535,8 +677,13 @@ mod tests {
                     Trailing text";
         let start = text.find("```\n").unwrap() + 4;
         let end = text.rfind("```\n").unwrap() + 4;
-        assert_eq!(CodeBlock::new(text, start, end),
-                   CodeBlock { content: "foo\n", first_line: 3 });
+        assert_eq!(
+            CodeBlock::new(text, start, end),
+            CodeBlock {
+                content: "foo\n",
+                first_line: 3
+            }
+        );
     }
 
     #[test]
@@ -611,32 +758,40 @@ mod tests {
         fn bad_major() {
             let version = parse_version("2.0.0").unwrap();
             let request = parse_request("1.2.3").unwrap();
-            assert_eq!(version_matches_request(&version, &request),
-                       Err(String::from("expected major version 2, found 1")));
+            assert_eq!(
+                version_matches_request(&version, &request),
+                Err(String::from("expected major version 2, found 1"))
+            );
         }
 
         #[test]
         fn bad_minor() {
             let version = parse_version("1.3.0").unwrap();
             let request = parse_request("1.2.3").unwrap();
-            assert_eq!(version_matches_request(&version, &request),
-                       Err(String::from("expected minor version 3, found 2")));
+            assert_eq!(
+                version_matches_request(&version, &request),
+                Err(String::from("expected minor version 3, found 2"))
+            );
         }
 
         #[test]
         fn bad_patch() {
             let version = parse_version("1.2.4").unwrap();
             let request = parse_request("1.2.3").unwrap();
-            assert_eq!(version_matches_request(&version, &request),
-                       Err(String::from("expected patch version 4, found 3")));
+            assert_eq!(
+                version_matches_request(&version, &request),
+                Err(String::from("expected patch version 4, found 3"))
+            );
         }
 
         #[test]
         fn bad_pre_release() {
             let version = parse_version("1.2.3-rc2").unwrap();
             let request = parse_request("1.2.3-rc1").unwrap();
-            assert_eq!(version_matches_request(&version, &request),
-                       Err(String::from("expected pre-release \"rc2\", found \"rc1\"")));
+            assert_eq!(
+                version_matches_request(&version, &request),
+                Err(String::from("expected pre-release \"rc2\", found \"rc1\""))
+            );
         }
     }
 
@@ -648,8 +803,7 @@ mod tests {
             let block = "[dependencies]\n\
                          foobar = '1.5'";
             let request = extract_version_request("foobar", block);
-            assert_eq!(request.unwrap().predicates,
-                       parse_request("1.5").unwrap().predicates);
+            assert_eq!(request.unwrap(), parse_request("1.5").unwrap());
         }
 
         #[test]
@@ -657,8 +811,17 @@ mod tests {
             let block = "[dependencies]\n\
                          foobar = { version = '1.5', default-features = false }";
             let request = extract_version_request("foobar", block);
-            assert_eq!(request.unwrap().predicates,
-                       parse_request("1.5").unwrap().predicates);
+            assert_eq!(request.unwrap(), parse_request("1.5").unwrap());
+        }
+
+        #[test]
+        fn git_dependency() {
+            // Git dependencies are translated into a "*" dependency
+            // and are thus always accepted.
+            let block = "[dependencies]\n\
+                         foobar = { git = 'https://example.net/foobar.git' }";
+            let request = extract_version_request("foobar", block);
+            assert_eq!(request.unwrap(), parse_request("*").unwrap());
         }
 
         #[test]
@@ -666,8 +829,7 @@ mod tests {
             let block = "[dev-dependencies]\n\
                          foobar = '1.5'";
             let request = extract_version_request("foobar", block);
-            assert_eq!(request.unwrap().predicates,
-                       parse_request("1.5").unwrap().predicates);
+            assert_eq!(request.unwrap(), parse_request("1.5").unwrap());
         }
 
         #[test]
@@ -675,8 +837,11 @@ mod tests {
             let block = "[dependencies]\n\
                          foobar = '1.5.bad'";
             let request = extract_version_request("foobar", block);
-            assert_eq!(request.unwrap_err(),
-                       "could not parse dependency: Extra junk after valid predicate: .bad");
+            assert_eq!(
+                request.unwrap_err(),
+                "could not parse dependency: \
+                 encountered unexpected token: AlphaNumeric(\"bad\")"
+            );
         }
 
         #[test]
@@ -698,8 +863,10 @@ mod tests {
             let block = "[dependencies]\n\
                          foobar = 1.5.8";
             let request = extract_version_request("foobar", block);
-            assert_eq!(request.unwrap_err(),
-                       "TOML parse error: expected newline, found a period at line 2");
+            assert_eq!(
+                request.unwrap_err(),
+                "TOML parse error: expected newline, found a period at line 2"
+            );
         }
     }
 
@@ -718,14 +885,24 @@ mod tests {
 
         #[test]
         fn single() {
-            assert_eq!(find_toml_blocks("```toml\n```"),
-                       vec![CodeBlock { content: "", first_line: 2 }]);
+            assert_eq!(
+                find_toml_blocks("```toml\n```"),
+                vec![CodeBlock {
+                    content: "",
+                    first_line: 2
+                }]
+            );
         }
 
         #[test]
         fn no_close_fence() {
-            assert_eq!(find_toml_blocks("```toml\n"),
-                       vec![CodeBlock { content: "", first_line: 2 }]);
+            assert_eq!(
+                find_toml_blocks("```toml\n"),
+                vec![CodeBlock {
+                    content: "",
+                    first_line: 2
+                }]
+            );
         }
     }
 
@@ -735,15 +912,19 @@ mod tests {
         #[test]
         fn good_url() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/foo/1.2.3", "foo", &ver),
-                       Ok(()));
+            assert_eq!(
+                url_matches("https://docs.rs/foo/1.2.3", "foo", &ver),
+                Ok(())
+            );
         }
 
         #[test]
         fn trailing_slash() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/foo/1.2.3/", "foo", &ver),
-                       Ok(()));
+            assert_eq!(
+                url_matches("https://docs.rs/foo/1.2.3/", "foo", &ver),
+                Ok(())
+            );
         }
 
         #[test]
@@ -767,72 +948,94 @@ mod tests {
         #[test]
         fn different_domain_http() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("http://example.net/foo/1.2.3", "foo", &ver),
-                       Ok(()));
+            assert_eq!(
+                url_matches("http://example.net/foo/1.2.3", "foo", &ver),
+                Ok(())
+            );
         }
 
         #[test]
         fn http_url() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("http://docs.rs/foo/1.2.3", "foo", &ver),
-                       Err(String::from("expected \"https\", found \"http\"")));
+            assert_eq!(
+                url_matches("http://docs.rs/foo/1.2.3", "foo", &ver),
+                Err(String::from("expected \"https\", found \"http\""))
+            );
         }
 
         #[test]
         fn bad_scheme() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("mailto:foo@example.net", "foo", &ver),
-                       Err(String::from("expected \"https\", found \"mailto\"")));
+            assert_eq!(
+                url_matches("mailto:foo@example.net", "foo", &ver),
+                Err(String::from("expected \"https\", found \"mailto\""))
+            );
         }
 
         #[test]
         fn no_package() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs", "foo", &ver),
-                       Err(String::from("missing package name")));
+            assert_eq!(
+                url_matches("https://docs.rs", "foo", &ver),
+                Err(String::from("missing package name"))
+            );
         }
 
         #[test]
         fn no_package_trailing_slash() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/", "foo", &ver),
-                       Err(String::from("missing package name")));
+            assert_eq!(
+                url_matches("https://docs.rs/", "foo", &ver),
+                Err(String::from("missing package name"))
+            );
         }
 
         #[test]
         fn no_version() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/foo", "foo", &ver),
-                       Err(String::from("missing version number")));
+            assert_eq!(
+                url_matches("https://docs.rs/foo", "foo", &ver),
+                Err(String::from("missing version number"))
+            );
         }
 
         #[test]
         fn no_version_trailing_slash() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/foo/", "foo", &ver),
-                       Err(String::from("missing version number")));
+            assert_eq!(
+                url_matches("https://docs.rs/foo/", "foo", &ver),
+                Err(String::from("missing version number"))
+            );
         }
 
         #[test]
         fn bad_url() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("docs.rs/foo/bar", "foo", &ver),
-                       Err(String::from("parse error: relative URL without a base")));
+            assert_eq!(
+                url_matches("docs.rs/foo/bar", "foo", &ver),
+                Err(String::from("parse error: relative URL without a base"))
+            );
         }
 
         #[test]
         fn bad_pkg_version() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/foo/1.2.bad/", "foo", &ver),
-                       Err(String::from("could not parse version in URL: \
-                                         Extra junk after valid predicate: .bad")));
+            assert_eq!(
+                url_matches("https://docs.rs/foo/1.2.bad/", "foo", &ver),
+                Err(String::from(
+                    "could not parse version in URL: \
+                     encountered unexpected token: AlphaNumeric(\"bad\")"
+                ))
+            );
         }
 
         #[test]
         fn wrong_pkg_name() {
             let ver = parse_version("1.2.3").unwrap();
-            assert_eq!(url_matches("https://docs.rs/foo/1.2.3/", "bar", &ver),
-                       Err(String::from("expected package \"bar\", found \"foo\"")));
+            assert_eq!(
+                url_matches("https://docs.rs/foo/1.2.3/", "bar", &ver),
+                Err(String::from("expected package \"bar\", found \"foo\""))
+            );
         }
     }
 
@@ -847,16 +1050,21 @@ mod tests {
                 "The system cannot find the file specified. (os error 2)"
             };
             let errmsg = format!("could not read no-such-file.md: {}", no_such_file);
-            assert_eq!(check_markdown_deps("no-such-file.md", "foobar", "1.2.3"),
-                       Err(errmsg));
+            assert_eq!(
+                check_markdown_deps("no-such-file.md", "foobar", "1.2.3"),
+                Err(errmsg)
+            );
         }
 
         #[test]
         fn bad_pkg_version() {
             // This uses the README.md file from this crate.
-            assert_eq!(check_markdown_deps("README.md", "foobar", "1.2"),
-                       Err(String::from("bad package version \"1.2\": \
-                                         Expected dot")));
+            assert_eq!(
+                check_markdown_deps("README.md", "foobar", "1.2"),
+                Err(String::from(
+                    "bad package version \"1.2\": expected more input"
+                ))
+            );
         }
     }
 
@@ -871,16 +1079,62 @@ mod tests {
                 "The system cannot find the file specified. (os error 2)"
             };
             let errmsg = format!("could not read no-such-file.md: {}", no_such_file);
-            assert_eq!(check_html_root_url("no-such-file.md", "foobar", "1.2.3"),
-                       Err(errmsg));
+            assert_eq!(
+                check_html_root_url("no-such-file.md", "foobar", "1.2.3"),
+                Err(errmsg)
+            );
         }
 
         #[test]
         fn bad_pkg_version() {
             // This uses the src/lib.rs file from this crate.
-            assert_eq!(check_html_root_url("src/lib.rs", "foobar", "1.2"),
-                       Err(String::from("bad package version \"1.2\": \
-                                         Expected dot")));
+            assert_eq!(
+                check_html_root_url("src/lib.rs", "foobar", "1.2"),
+                Err(String::from(
+                    "bad package version \"1.2\": expected more input"
+                ))
+            );
         }
     }
+
+    mod test_check_contains_regex {
+        use super::*;
+
+        #[test]
+        fn bad_regex() {
+            // Check that the error from a bad pattern doesn't contain
+            // the (?m) prefix.
+            assert_eq!(
+                check_contains_regex("README.md", "Version {version} [ups", "foobar", "1.2.3"),
+                Err(String::from(
+                    [
+                        "could not parse template: regex parse error:",
+                        "    Version 1.2.3 [ups",
+                        "                  ^",
+                        "error: unclosed character class"
+                    ]
+                        .join("\n")
+                ))
+            )
+        }
+
+        #[test]
+        fn not_found() {
+            assert_eq!(
+                check_contains_regex("README.md", "should not be found", "foobar", "1.2.3"),
+                Err(String::from(
+                    "could not find \"should not be found\" in README.md"
+                ))
+            )
+        }
+
+        #[test]
+        fn good_pattern() {
+            assert_eq!(
+                check_contains_regex("README.md", "{name}", "version-sync", "1.2.3"),
+                Ok(())
+            )
+        }
+
+    }
 }
